/**
 * Kosaraju
 *
 * Complexity (Time): O(V + E)
 * Complexity (Space): O(V + E)
 */

stack<int> S;
vector<int> graph[MAX];
vector<int> transp[MAX];

bool cont[MAX];

/**
 * Traverses a SCC.
 * @param x initial vertex
 */
void dfs(int x) {
  // x belong to current scc
  cont[x] = true;

  for (auto i : transp[x])
    if (!cont[i])
      dfs(i);
}

/**
 * Fills stack with DFS starting points to find SCC.
 * @param x initial vertex
 */
void fill_stack(int x) {
  cont[x] = true;

  for (auto i : graph[x])
    if (!cont[i])
      fill_stack(i);

  S.push(x);
}

/**
 * Returns number of SCC of a graph.
 * @param n number of vertices
 */
int kosaraju(int n) {
  int scc = 0;

  mset(cont, 0);
  for (int i = 0; i < n; ++i)
    if (!cont[i])
      fill_stack(i);

  // Transpose graph
  for (int i = 0; i < n; ++i)
    for (auto j : graph[i])
      transp[j].push_back(i);

  // Count SCC
  mset(cont, 0);
  while (!S.empty()) {
    int v = S.top();
    S.pop();

    if (!cont[v]) {
      dfs(v);
      scc++;
    }
  }

  return scc;
}
