/**
 * Kruskal
 *
 * Complexity (Time): O (E log V) 
 * Complexity (Space): O(E)
 * 
 * #include <structure/disjoint_set>
 *
 * OBS: * = return maximum spanning tree
 */

typedef pair<ii,int> iii;
vector<iii> edges;

/**
 * Returns value of MST.
 * @param mst[out] vector with edges of computed MST
 */
int kruskal(vector<iii> &mst) {

  // Sort by weight of the edges
  sort(all(edges), [&](const iii &a, const iii &b) {
    return a.se < b.se;
    //* return a.se > b.se
  });

  int size = 0;
  for (int i = 0; i < MAX; i++)
    make_set(i);

  for (int i = 0; i < edges.size(); i++) {
    int pu = find_set(edges[i].fi.fi);
    int pv = find_set(edges[i].fi.se);

    // If the sets are different, then the edge i does not close a cycle
    if (pu != pv) {
      mst.pb(edges[i]);
      size += edges[i].se;
      union_set(pu, pv);
    }
  }

  return size;
}
