/**
 * Topological Sort
 *
 * Complexity (Time): O(V + E)
 * Complexity (Space): O(V + E)
 */

stack<int> S;
vector<int> graph[MAX];

int cont[MAX];

/**
 * Fills stack and check for cycles.
 * @param x any vertex
 */
bool dfs(int x) {
  cont[x] = 1;

  for (auto i : graph[x]) {
    if (cont[i] == 1)
      return true;
    if (!cont[i] && dfs(i))
      return true;
  }

  cont[x] = 2;
  S.push(x);

  return false;
}

/** 
 * Returns whether graph contains cycle or not.
 * @param n number of vertices
 * @param[out] tsort topological sort of the graph
 */
bool topological_sort(int n, vector<int> &tsort) {
  mset(cont, 0);
  
  bool cycle = false;
  for (int i = 0; i < n; ++i)
    if (!cont[i])
      cycle |= dfs(i);

  if (cycle)
    return true;

  while (!S.empty()) {
    tsort.pb(S.top());
    S.pop();
  }

  return false;
}
