/**
 * Linear Recurrence
 *
 * Complexity (Time): O(log n)
 * Complexity (Space): O(1)
 * 
 * #include "math/binary_exponentiation.cpp"
 * #include "math/matrix.cpp"
 */

/**
 * Solves f(n) = x * f(n - 1) + y * f(n - 2)
 * This algorithm is used to solve recurrences such as:
 *   f(n) = x1 * f(n - 1) + x2 * f(n - 1) + ... + xk * f(n - k)
 *
 * It works by defining this recurrence as a linear combination,
 * for example (k = 2):
 *   f(n) = [x1  x2] [f(n - 1)]
 *                   [f(n - 2)]
 * It can be rewriten as:
 *   [  f(n)  ] = [x1  x2] [f(n - 1)]
 *   [f(n - 1)]   [ 1   0] [f(n - 2)]
 *
 * And that is solved by calculating the following matrix power:
 *   [x1  x2]^n
 *   [ 1   0]
 */
matrix solve(ll x, ll y, ll n) {
  matrix in(2);

  in[0][0] = x % MOD;
  in[0][1] = y % MOD;
  in[1][0] = 1;
  in[1][1] = 0;

  return fast_pow(in, n);
}
