/**
 * Sieve of Eratosthenes 
 *
 * Complexity (Time): O(n*log(log(n)))
 * Complexity (Space): O(n)
 */

/**
 * Returns vector of primes less than or equal to n
 */
vector<int> sieve(int n) {
  vector<int> primes;
  vector<bool> is_prime(n+1, true);

  // Mark which elements are not prime (multiples of p)
  for (int p = 2; p*p <= n; ++p)
    if (is_prime[p])
      for (int i = p*p; i <= n; i += p)
        is_prime[i] = false;

  // Add non-marked elements (primes) to the list
  for (int p = 2; p <= n; ++p)
    if (is_prime[p])
      primes.pb(p);

  return primes;
}
