/**
 * Knuth-Morris-Pratt (KMP)
 *
 * Complexity (Time): 
 *   preprocess -> O(m)
 *   search     -> O(n)
 * Complexity (Space): O(n + m)
 */

int table[MAX];

/**
 * Builds the table where table[i] is the longest prefix of patt[0..i] which is
 * also a sufix of patt[0..i]
 * @param patt pattern to be found
 */
void preprocess(string patt) {
  int i = 1, len = 0;

  while (i < patt.size()) {
    if (patt[i] == patt[len])
      table[i++] = ++len;
    else if (len)
      len = table[len - 1];
    else
      table[i++] = 0;
  }
}

/**
 * Searches for occurrences of patt in txt and add indexes of matches to occurs
 * @param patt pattern to be found
 * @param txt text
 */
void search(string patt, string txt) {
  int i = 0, j = 0;
  vector<int> occurs;

  while (i < txt.size()) {
    if (patt[j] == txt[i])
      i++, j++;

    if (j == patt.size()) {
      // Pattern found at (i - j)
      occurs.push_back(i - j);
      j = table[j - 1];
    } else if (i < txt.size() && patt[j] != txt[i]) {
      if (j > 0) 
        j = table[j - 1];
      else 
        i++;
    }
  }
}
