/**
 * Bitmask 
 *
 * Complexity (Time): O(1)
 * Complexity (Space): O(1)
 */

/**
 * Sets bit in position pos (0 to 1).
 */
void set(ll &bitmask, int pos) { 
  bitmask |= (1 << pos); 
}

/**
 * Sets all bits in a bitmask with size n.
 */
void set_all(ll &bitmask, int n) { 
  bitmask = (1 << n) - 1; 
}

/**
 * Unsets bit in position pos (1 to 0).
 */
void unset(ll &bitmask, int pos) { 
  bitmask &= ~(1 << pos); 
}

/**
 * Unsets all bits.
 */
void unset_all(ll &bitmask) { 
  bitmask = 0; 
}

/**
 * Gets value of bit in position pos.
 */
int get(ll bitmask, int pos) { 
  return bitmask & (1 << pos); 
}

/**
 * Toggles value in position pos.
 */
void toggle(ll &bitmask, int pos) { 
  bitmask ^= (1 << pos); 
}

/**
 * Gets position of least significant 1.
 */
int least_significant_one(ll bitmask) {
  return bitmask & (-bitmask); 
}
