/**
 * Segment Tree
 *
 * Complexity (Time):
 *   Build  -> O(n log n)
 *   Update -> O(log n)
 *   Query  -> O(log n)
 * Complexity (Space): O(n)
 */

int N;
int v[MAX];
int tree[4 * MAX];

#define left(x) (x << 1)
#define right(x) ((x << 1) + 1)

/**
 * Builds tree with elements from v (0-indexed).
 */
void build(int node = 1, int a = 0, int b = N - 1) {
  if (a > b) 
    return;

  if (a == b) {
    tree[node] = v[a];
    return;
  }

  build_tree(left(node), a, (a + b) / 2);
  build_tree(right(node), 1 + (a + b) / 2, b);
  tree[node] = tree[node * 2] + tree[node * 2 + 1];
}

/**
 * Updates position idx by adding value val.
 * @param idx position
 * @param val value to be added
 */
void update(int idx, int val, int node = 1, int a = 0, int b = N - 1) {
  if (a > b || a > idx || b < idx) 
    return;

  if (a == b) {
    tree[node] = val;
    return;
  }

  update_tree(idx, val, left(node), a, (a + b) / 2);
  update_tree(idx, val, right(node), 1 + (a + b) / 2, b);
  tree[node] = tree[node * 2] + tree[node * 2 + 1];
}

/**
 * Returns sum of [i,j].
 * @param i,j range
 */
int query(int i, int j, int node = 1, int a = 0, int b = N - 1) {
  if (a > b || a > j || b < i) 
    return 0;

  if (i <= a && b <= j) 
    return tree[node];

  int q1 = query_tree(i, j, left(node), a, (a + b) / 2);
  int q2 = query_tree(i, j, right(node), 1 + (a + b) / 2, b);
  return q1 + q2;
}
