/**
 * Trie 
 *
 * Complexity (Time):
 *   Insert -> O(m)
 *   Search -> O(m)
 * Complexity (Space): O(alphabet_size * N)
 */

// ========== Trie for String ==========
int states = 0;
int trie[MAX][26]; // mset(-1)

// ending[i] is true when the node i represents the last
// character of a string contained in the Trie
bool ending[MAX];

/**
 * Inserts word into the Trie.
 */
void insert(string word) {
  int node = 0;

  for (int i = 0; i < word.size(); ++i) {
    int b = word[i] - 'a';

    if (trie[node][b] == -1)
      trie[node][b] = ++states;
    node = trie[node][b];
  }

  ending[node] = true;
}

/**
 * Returns true if word is in the Trie.
 */
bool search(string word) {
  int node = 0;

  for (int i = 0; i < word.size(); ++i) {
    node = trie[node][word[i] - 'a'];
    if (node == -1)
      return false;
  }

  return ending[node];
}


// ========== Trie for Integer ==========
int states = 0;
int trie[MAX][2]; // mset(-1)

// ending[i] is true when the node i represents the last
// bit of an integer contained in the Trie
bool ending[MAX];

/**
 * Inserts x into the Trie.
 */
void insert(int x) {
  int node = 0;

  for (int i = 30; i >= 0; --i) {
    int b = !!(x & (1 << i));

    if (trie[node][b] == -1)
      trie[node][b] = ++states;
    node = trie[node][b];
  }

  ending[node] = true;
}

/** 
 * Returns true if x is in the Trie.
 */
bool search(int x) {
  int node = 0;
  
  for (int i = 30; i >= 0; --i) {
    int b = !!(x & (1 << i));
    node = trie[node][b];
    if (node == -1)
      return false;
  }

  return ending[node];
}
